@@ -151,6 +151,10 @@ class ValidateInterfacesHook(base.ProcessingHook):
 
         pxe_mac = introspection_data.get('boot_interface')
 
+        if CONF.processing.add_ports == 'pxe' and not pxe_mac:
+            LOG.warning(_LW('No boot interface provided in the introspection '
+                            'data, will add all ports with IP addresses'))
+
         if CONF.processing.add_ports == 'pxe' and pxe_mac:
             LOG.info(_LI('PXE boot interface was %s'), pxe_mac)
             if '-' in pxe_mac:
@@ -231,9 +235,10 @@ class RamdiskErrorHook(base.ProcessingHook):
 
     def _store_logs(self, logs, introspection_data):
         if not CONF.processing.ramdisk_logs_dir:
-            LOG.warn(_LW('Failed to store logs received from the ramdisk '
-                         'because ramdisk_logs_dir configuration option '
-                         'is not set'))
+            LOG.warning(
+                _LW('Failed to store logs received from the ramdisk '
+                    'because ramdisk_logs_dir configuration option '
+                    'is not set'))
             return
 
         if not os.path.exists(CONF.processing.ramdisk_logs_dir):
