@@ -81,13 +81,14 @@ def process(introspection_data):
             LOG.exception(_LE('Hook %(hook)s failed, delaying error report '
                               'until node look up: %(error)s'),
                           {'hook': hook_ext.name, 'error': exc})
-            failures.append(_('Unexpected exception %(exc_class)s during '
-                              'preprocessing in hook %(hook)s: %(error)s') %
-                            {'hook': hook_ext.name,
-                             'exc_class': exc.__class__.__name__,
-                             'error': exc})
+            failures.append(_('Unexpected exception during preprocessing '
+                              'in hook %s') % hook_ext.name)
 
     node_info = _find_node_info(introspection_data, failures)
+    if node_info:
+        # Locking is already done in find_node() but may be not done in a
+        # node_not_found hook
+        node_info.acquire_lock()
 
     if failures and node_info:
         msg = _('The following failures happened during running '
@@ -95,7 +96,7 @@ def process(introspection_data):
             'uuid': node_info.uuid,
             'failures': '\n'.join(failures)
         }
-        node_info.finished(error='\n'.join(failures))
+        node_info.finished(error=_('Data pre-processing failed'))
         raise utils.Error(msg)
     elif not node_info:
         msg = _('The following failures happened during running '
@@ -118,10 +119,8 @@ def process(introspection_data):
         node_info.finished(error=str(exc))
         raise
     except Exception as exc:
-        LOG.exception(_LE('Unexpected exception during processing'))
-        msg = _('Unexpected exception %(exc_class)s during processing: '
-                '%(error)s') % {'exc_class': exc.__class__.__name__,
-                                'error': exc}
+        msg = _('Unexpected exception during processing')
+        LOG.exception(msg)
         node_info.finished(error=msg)
         raise utils.Error(msg)
 
@@ -136,11 +135,11 @@ def _run_post_hooks(node_info, introspection_data):
                                    node_patches=node_patches,
                                    ports_patches=ports_patches)
         if node_patches:
-            LOG.warn(_LW('Using node_patches is deprecated'))
+            LOG.warning(_LW('Using node_patches is deprecated'))
             node_info.patch(node_patches)
 
         if ports_patches:
-            LOG.warn(_LW('Using ports_patches is deprecated'))
+            LOG.warning(_LW('Using ports_patches is deprecated'))
             for mac, patches in ports_patches.items():
                 node_info.patch_port(mac, patches)
 
